{%- set tu = task_unit if task_unit else {} -%}
{%- set statuses = tu.get('statuses', ['TODO', 'IN_PROGRESS', 'REVIEW', 'DONE']) -%}
# 四、开发流程协议

## 4.1 任务单元定义

开发不按日期，按 **对话任务单元** 推进：

```
任务单元 (Task Unit):
├── ID: {{ tu.get('id_pattern', 'TASK-{role}-{seq}') }}
{% for field in tu.get('required_fields', []) if field != 'id' -%}
├── {{ field }}
{% endfor -%}
└── 状态: {{ statuses | join(' / ') }}
```
