{%- set lc = config.get('lifecycle', {}) -%}
{%- set stages = lc.get('stages', {}) -%}
{%- if not stages -%}
{%- set stages = {
    'demo': {
        'name': '原型验证',
        'description': '快速验证核心概念和可行性',
        'focus': ['快速迭代', '概念验证', '核心功能'],
        'principles': [
            '快速试错，快速调整',
            '优先核心功能，暂缓优化',
            '技术债务可接受，但需记录',
            '详细的Git开发迭代记录',
            '记录重要决定DECISIONS.md',
            '建立 CI/CD'
        ]
    },
    'production': {
        'name': '量产',
        'description': '产品化开发，准备规模化',
        'focus': ['稳定性', '性能优化', '可维护性'],
        'principles': [
            '代码质量优先',
            '建立发布和宣发预备, 指定和完善目标平台支持',
            '启动前review全量代码，建立更稳定稳健的代码结构',
            '完善QA产品测试覆盖',
            '定义性能标准',
            'Unitest单元测试、检查规范',
            '完善发布平台标准'
        ]
    },
    'commercial': {
        'name': '商业化',
        'description': '面向市场，追求增长',
        'focus': ['用户体验', '市场适配', '扩展性', '插件化增量开发', '数据热更'],
        'principles': [
            '用户反馈驱动',
            '数据驱动决策',
            '快速响应市场'
        ]
    },
    'stable': {
        'name': '稳定运营',
        'description': '成熟产品，稳定维护',
        'focus': ['稳定性', '维护成本', '长期规划'],
        'principles': [
            '变更需谨慎',
            '向后兼容优先',
            '文档完善'
        ]
    }
} -%}
{%- endif -%}
{%- set stage_order = ['demo', 'production', 'commercial', 'stable'] -%}
# 八、阶段化协作规则

## 8.1 项目生涯阶段说明

项目开发过程分为 4 个生涯阶段，每个阶段有不同的开发重点和协作原则。阶段信息由 PM 角色在 `docs/ROADMAP.md` 中维护，AI 在协作时应根据当前阶段调整工作方式。

### 阶段类型定义

项目生涯阶段按顺序演进，每个阶段都有明确的定义和规则：

{% for stage_code in stage_order -%}
{%- set stage_data = stages.get(stage_code, {}) -%}
{% if stage_data -%}
### {{ stage_data.get('name', stage_code) }} ({{ stage_code }})

**描述**: {{ stage_data.get('description', '') }}

**阶段重点**:
{% for focus in stage_data.get('focus', []) -%}
- {{ focus }}
{% endfor %}
**阶段原则**:
{% for principle in stage_data.get('principles', []) -%}
- {{ principle }}
{% endfor %}
{% endif -%}
{% endfor -%}
## 8.2 阶段化协作指导

AI 在协作时应：

1. **读取当前阶段**: 在对话开始时，读取 `docs/ROADMAP.md` 了解项目当前处于哪个阶段
2. **应用阶段规则**: 根据当前阶段的重点和原则调整工作方式
3. **关注阶段变化**: 当项目升级到新阶段时，注意新阶段的原则变化，调整协作方式
4. **阶段里程碑**: 关注当前阶段的里程碑完成情况，协助推进里程碑达成

> **重要**: 具体的当前阶段信息请查看 `docs/ROADMAP.md` 中的"当前项目生涯阶段"章节。

---
